@@ -131,7 +131,7 @@ class Pilot(Generic[ReturnType]):
         """
         try:
             return await self._post_mouse_events(
-                [MouseDown],
+                [MouseMove, MouseDown],
                 widget=widget,
                 offset=offset,
                 button=1,
@@ -176,7 +176,7 @@ class Pilot(Generic[ReturnType]):
         """
         try:
             return await self._post_mouse_events(
-                [MouseUp],
+                [MouseMove, MouseUp],
                 widget=widget,
                 offset=offset,
                 button=1,
@@ -442,7 +442,8 @@ class Pilot(Generic[ReturnType]):
                 # the driver works and emits a click event.
                 kwargs = message_arguments
                 if mouse_event_cls is Click:
-                    kwargs["chain"] = chain
+                    kwargs = {**kwargs, "chain": chain}
+
                 widget_at, _ = app.get_widget_at(*offset)
                 event = mouse_event_cls(**kwargs)
                 # Bypass event processing in App.on_event. Because App.on_event
