@@ -1,13 +1,14 @@
 """
-A Strip contains the result of rendering a widget.
-See [line API](/guide/widgets#line-api) for how to use Strips.
-"""
+This module contains the `Strip` class and related objects.
 
+A `Strip` contains the result of rendering a widget.
+See [Line API](/guide/widgets#line-api) for how to use Strips.
+"""
 
 from __future__ import annotations
 
 from itertools import chain
-from typing import Iterable, Iterator, Sequence
+from typing import Any, Iterable, Iterator, Sequence
 
 import rich.repr
 from rich.cells import cell_len, set_cell_size
@@ -16,11 +17,12 @@ from rich.measure import Measurement
 from rich.segment import Segment
 from rich.style import Style, StyleType
 
-from ._cache import FIFOCache
-from ._segment_tools import index_to_cell_position
-from .color import Color
-from .constants import DEBUG
-from .filter import LineFilter
+from textual._segment_tools import index_to_cell_position, line_pad
+from textual.cache import FIFOCache
+from textual.color import Color
+from textual.constants import DEBUG
+from textual.css.types import AlignHorizontal, AlignVertical
+from textual.filter import LineFilter
 
 
 def get_line_length(segments: Iterable[Segment]) -> int:
@@ -37,7 +39,7 @@ def get_line_length(segments: Iterable[Segment]) -> int:
 
 
 class StripRenderable:
-    """A renderable which renders a list of strips in to lines."""
+    """A renderable which renders a list of strips into lines."""
 
     def __init__(self, strips: list[Strip], width: int | None = None) -> None:
         self._strips = strips
@@ -110,8 +112,11 @@ class Strip:
             assert get_line_length(self._segments) == cell_length
 
     def __rich_repr__(self) -> rich.repr.Result:
-        yield self._segments
-        yield self.cell_length
+        try:
+            yield self._segments
+            yield self.cell_length
+        except AttributeError:
+            pass
 
     @property
     def text(self) -> str:
@@ -156,6 +161,102 @@ class Strip:
         """
         return [cls(segments, cell_length) for segments in lines]
 
+    @classmethod
+    def align(
+        cls,
+        strips: list[Strip],
+        style: Style,
+        width: int,
+        height: int | None,
+        horizontal: AlignHorizontal,
+        vertical: AlignVertical,
+    ) -> Iterable[Strip]:
+        """Align a list of strips on both axis.
+
+        Args:
+            strips: A list of strips, such as from a render.
+            style: The Rich style of additional space.
+            width: Width of container.
+            height: Height of container.
+            horizontal: Horizontal alignment method.
+            vertical: Vertical alignment method.
+
+        Returns:
+            An iterable of strips, with additional padding.
+
+        """
+        if not strips:
+            return
+        line_lengths = [strip.cell_length for strip in strips]
+        shape_width = max(line_lengths)
+        shape_height = len(line_lengths)
+
+        def blank_lines(count: int) -> Iterable[Strip]:
+            """Create blank lines.
+
+            Args:
+                count: Desired number of blank lines.
+
+            Returns:
+                An iterable of blank lines.
+            """
+            blank = cls([Segment(" " * width, style)], width)
+            for _ in range(count):
+                yield blank
+
+        top_blank_lines = bottom_blank_lines = 0
+        if height is not None:
+            vertical_excess_space = max(0, height - shape_height)
+
+            if vertical == "top":
+                bottom_blank_lines = vertical_excess_space
+            elif vertical == "middle":
+                top_blank_lines = vertical_excess_space // 2
+                bottom_blank_lines = vertical_excess_space - top_blank_lines
+            elif vertical == "bottom":
+                top_blank_lines = vertical_excess_space
+
+            if top_blank_lines:
+                yield from blank_lines(top_blank_lines)
+
+        if horizontal == "left":
+            for strip in strips:
+                if strip.cell_length == width:
+                    yield strip
+                else:
+                    yield Strip(
+                        line_pad(strip._segments, 0, width - strip.cell_length, style),
+                        width,
+                    )
+        elif horizontal == "center":
+            left_space = max(0, width - shape_width) // 2
+            for strip in strips:
+                if strip.cell_length == width:
+                    yield strip
+                else:
+                    yield Strip(
+                        line_pad(
+                            strip._segments,
+                            left_space,
+                            width - strip.cell_length - left_space,
+                            style,
+                        ),
+                        width,
+                    )
+
+        elif horizontal == "right":
+            for strip in strips:
+                if strip.cell_length == width:
+                    yield strip
+                else:
+                    yield cls(
+                        line_pad(strip._segments, width - strip.cell_length, 0, style),
+                        width,
+                    )
+
+        if bottom_blank_lines:
+            yield from blank_lines(bottom_blank_lines)
+
     def index_to_cell_position(self, index: int) -> int:
         """Given a character index, return the cell position of that character.
         This is the sum of the cell lengths of all the characters *before* the character
@@ -179,7 +280,7 @@ class Strip:
 
     @classmethod
     def join(cls, strips: Iterable[Strip | None]) -> Strip:
-        """Join a number of strips in to one.
+        """Join a number of strips into one.
 
         Args:
             strips: An iterable of Strips.
@@ -243,6 +344,9 @@ class Strip:
             A new strip with the supplied cell length.
         """
 
+        if self.cell_length == cell_length:
+            return self
+
         cache_key = (cell_length, style)
         cached_strip = self._line_length_cache.get(cache_key)
         if cached_strip is not None:
@@ -282,7 +386,6 @@ class Strip:
             strip = self
 
         self._line_length_cache[cache_key] = strip
-
         return strip
 
     def simplify(self) -> Strip:
@@ -297,6 +400,34 @@ class Strip:
         )
         return line
 
+    def discard_meta(self) -> Strip:
+        """Remove all meta from segments.
+
+        Returns:
+            New strip.
+        """
+
+        def remove_meta_from_segment(segment: Segment) -> Segment:
+            """Build a Segment with no meta.
+
+            Args:
+                segment: Segment.
+
+            Returns:
+                Segment, sans meta.
+            """
+            text, style, control = segment
+            if style is None:
+                return segment
+            style = style.copy()
+            style._meta = None
+            return Segment(text, style, control)
+
+        return Strip(
+            [remove_meta_from_segment(segment) for segment in self._segments],
+            self._cell_length,
+        )
+
     def apply_filter(self, filter: LineFilter, background: Color) -> Strip:
         """Apply a filter to all segments in the strip.
 
@@ -357,7 +488,7 @@ class Strip:
         cached_result = self._crop_extend_cache.get(cache_key)
         if cached_result is not None:
             return cached_result
-        strip = self.extend_cell_length(end).crop(start, end)
+        strip = self.extend_cell_length(end, style).crop(start, end)
         self._crop_extend_cache[cache_key] = strip
         return strip
 
@@ -371,10 +502,13 @@ class Strip:
         Returns:
             A new Strip.
         """
+
         start = max(0, start)
         end = self.cell_length if end is None else min(self.cell_length, end)
         if start == 0 and end == self.cell_length:
             return self
+        if end <= start:
+            return Strip([], 0)
         cache_key = (start, end)
         cached = self._crop_cache.get(cache_key)
         if cached is not None:
@@ -385,7 +519,7 @@ class Strip:
         add_segment = output_segments.append
         iter_segments = iter(self._segments)
         segment: Segment | None = None
-        if start > self.cell_length:
+        if start >= self.cell_length:
             strip = Strip([], 0)
         else:
             for segment in iter_segments:
@@ -417,7 +551,7 @@ class Strip:
         return strip
 
     def divide(self, cuts: Iterable[int]) -> Sequence[Strip]:
-        """Divide the strip in to multiple smaller strips by cutting at given (cell) indices.
+        """Divide the strip into multiple smaller strips by cutting at given (cell) indices.
 
         Args:
             cuts: An iterable of cell positions as ints.
@@ -465,6 +599,40 @@ class Strip:
         self._style_cache[style] = styled_strip
         return styled_strip
 
+    def apply_meta(self, meta: dict[str, Any]) -> Strip:
+        """Apply meta to all segments.
+
+        Args:
+            meta: A dict of meta information.
+
+        Returns:
+            A new strip.
+
+        """
+        meta_style = Style.from_meta(meta)
+        return self.apply_style(meta_style)
+
+    def _apply_link_style(self, link_style: Style) -> Strip:
+        segments = self._segments
+        _Segment = Segment
+        segments = [
+            (
+                _Segment(
+                    text,
+                    (
+                        style
+                        if style._meta is None
+                        else (style + link_style if "@click" in style.meta else style)
+                    ),
+                    control,
+                )
+                if style
+                else _Segment(text)
+            )
+            for text, style, control in segments
+        ]
+        return Strip(segments, self._cell_length)
+
     def render(self, console: Console) -> str:
         """Render the strip into terminal sequences.
 
@@ -485,3 +653,83 @@ class Strip:
                 ]
             )
         return self._render_cache
+
+    def crop_pad(self, cell_length: int, left: int, right: int, style: Style) -> Strip:
+        """Crop the strip to `cell_length`, and add optional padding.
+
+        Args:
+            cell_length: Cell length of strip prior to padding.
+            left: Additional padding on the left.
+            right: Additional padding on the right.
+            style: Style of any padding.
+
+        Returns:
+            Cropped and padded strip.
+        """
+        if cell_length != self.cell_length:
+            strip = self.adjust_cell_length(cell_length, style)
+        else:
+            strip = self
+        if not (left or right):
+            return strip
+        segments = strip._segments.copy()
+        if left:
+            segments.insert(0, Segment(" " * left, style))
+        if right:
+            segments.append(Segment(" " * right, style))
+        return Strip(segments, cell_length + left + right)
+
+    def text_align(self, width: int, align: AlignHorizontal) -> Strip:
+        if align == "left":
+            if self.cell_length == width:
+                return self
+            else:
+                return Strip(
+                    line_pad(self._segments, 0, width - self.cell_length, Style.null()),
+                    width,
+                )
+        elif align == "center":
+            left_space = max(0, width - self.cell_length) // 2
+
+            if self.cell_length == width:
+                return self
+            else:
+                return Strip(
+                    line_pad(
+                        self._segments,
+                        left_space,
+                        width - self.cell_length - left_space,
+                        Style.null(),
+                    ),
+                    width,
+                )
+
+        elif align == "right":
+            if self.cell_length == width:
+                return self
+            else:
+                return Strip(
+                    line_pad(self._segments, width - self.cell_length, 0, Style.null()),
+                    width,
+                )
+
+    def apply_offsets(self, x: int, y: int) -> Strip:
+        """Apply offsets used in text selection.
+
+        Args:
+            x: Offset on X axis (column).
+            y: Offset on Y axis (row).
+
+        Returns:
+            New strip.
+        """
+        segments = self._segments
+        strip_segments: list[Segment] = []
+        for segment in segments:
+            text, style, _ = segment
+            offset_style = Style.from_meta({"offset": (x, y)})
+            strip_segments.append(
+                Segment(text, style + offset_style if style else offset_style)
+            )
+            x += len(segment.text)
+        return Strip(strip_segments, self._cell_length)
