@@ -6,12 +6,13 @@ from itertools import zip_longest
 from typing import Awaitable
 
 from rich.repr import Result
-from rich.text import Text, TextType
+from rich.text import TextType
 from typing_extensions import Final
 
 from textual import events
 from textual.app import ComposeResult
 from textual.await_complete import AwaitComplete
+from textual.content import ContentType
 from textual.css.query import NoMatches
 from textual.message import Message
 from textual.reactive import reactive
@@ -60,7 +61,9 @@ class ContentTab(Tab):
             else content_id
         )
 
-    def __init__(self, label: Text, content_id: str, disabled: bool = False) -> None:
+    def __init__(
+        self, label: ContentType, content_id: str, disabled: bool = False
+    ) -> None:
         """Initialize a ContentTab.
 
         Args:
@@ -202,7 +205,7 @@ class TabPane(Widget):
 
     def __init__(
         self,
-        title: TextType,
+        title: ContentType,
         *children: Widget,
         name: str | None = None,
         id: str | None = None,
@@ -312,7 +315,7 @@ class TabbedContent(Widget):
 
     def __init__(
         self,
-        *titles: TextType,
+        *titles: ContentType,
         initial: str = "",
         name: str | None = None,
         id: str | None = None,
