@@ -1,10 +1,9 @@
 from __future__ import annotations
 
 from dataclasses import dataclass
-from typing import ClassVar
+from typing import TYPE_CHECKING, ClassVar
 
 import rich.repr
-from rich.console import RenderableType
 from rich.style import Style
 from rich.text import Text, TextType
 
@@ -22,6 +21,9 @@ from textual.renderables.bar import Bar
 from textual.widget import Widget
 from textual.widgets import Static
 
+if TYPE_CHECKING:
+    from textual.content import Content, ContentType
+
 
 class Underline(Widget):
     """The animated underline beneath tabs."""
@@ -148,7 +150,7 @@ class Tab(Static):
 
     def __init__(
         self,
-        label: TextType,
+        label: ContentType,
         *,
         id: str | None = None,
         classes: str | None = None,
@@ -163,23 +165,23 @@ class Tab(Static):
             disabled: Whether the tab is disabled or not.
         """
         super().__init__(id=id, classes=classes, disabled=disabled)
-        self._label: Text
+        self._label: Content
         # Setter takes Text or str
         self.label = label  # type: ignore[assignment]
 
     @property
-    def label(self) -> Text:
+    def label(self) -> Content:
         """The label for the tab."""
         return self._label
 
     @label.setter
-    def label(self, label: TextType) -> None:
-        self._label = Text.from_markup(label) if isinstance(label, str) else label
+    def label(self, label: ContentType) -> None:
+        self._label = self.render_str(label)
         self.update(self._label)
 
-    def update(self, renderable: RenderableType = "") -> None:
+    def update(self, content: ContentType = "") -> None:
         self.post_message(self.Relabelled(self))
-        return super().update(renderable)
+        return super().update(self.render_str(content))
 
     @property
     def label_text(self) -> str:
@@ -524,7 +526,7 @@ class Tabs(Widget, can_focus=True):
         underline = self.query_one(Underline)
         underline.highlight_start = 0
         underline.highlight_end = 0
-        self.call_after_refresh(self.post_message, self.Cleared(self))
+        self.post_message(self.Cleared(self))
         self.active = ""
         return AwaitComplete(self.query("#tabs-list > Tab").remove())
 
