@@ -8,7 +8,9 @@ class InputApp(App):
     CSS = "Input { padding: 4 8 }"
 
     def compose(self) -> ComposeResult:
-        yield Input("こんにちは!")
+        # We don't want to select the text on focus, as selected text
+        # has different interactions with the cursor_left action.
+        yield Input("こんにちは!", select_on_focus=False)
 
 
 async def test_initial_terminal_cursor_position():
