@@ -10,9 +10,9 @@ def main():
 """
 
 
-class MarkdownThemeSwitchertApp(App[None]):
+class MarkdownThemeSwitcherApp(App[None]):
     BINDINGS = [
-        ("t", "toggle_dark"),
+        ("t", "toggle_theme"),
         ("d", "switch_dark"),
         ("l", "switch_light"),
     ]
@@ -25,9 +25,12 @@ class MarkdownThemeSwitchertApp(App[None]):
         md = self.query_one(Markdown)
         md.code_light_theme = "solarized-light"
 
+    def action_toggle_theme(self) -> None:
+        self.theme = "textual-light" if self.theme == "textual-dark" else "textual-dark"
+
     def compose(self) -> ComposeResult:
         yield Markdown(TEST_CODE_MARKDOWN)
 
 
 if __name__ == "__main__":
-    MarkdownThemeSwitchertApp().run()
+    MarkdownThemeSwitcherApp().run()
