@@ -3,8 +3,10 @@ from textual.widgets import ProgressBar
 
 
 class TooltipApp(App[None]):
+    TOOLTIP_DELAY = 0.4
+
     def compose(self) -> ComposeResult:
-        progress_bar = ProgressBar(100)
+        progress_bar = ProgressBar(100, show_eta=False)
         progress_bar.advance(10)
         progress_bar.tooltip = "Hello, Tooltip!"
         yield progress_bar
