@@ -1,6 +1,6 @@
 import pytest
 
-from textual.expand_tabs import expand_tabs_inline
+from textual.expand_tabs import expand_tabs_inline, get_tab_widths
 
 
 @pytest.mark.parametrize(
@@ -23,3 +23,10 @@ from textual.expand_tabs import expand_tabs_inline
 )
 def test_expand_tabs_inline(line, expanded_line):
     assert expand_tabs_inline(line) == expanded_line
+
+
+def test_get_tab_widths():
+    assert get_tab_widths("\tbar") == [("", 4), ("bar", 0)]
+    assert get_tab_widths("\tbar\t") == [("", 4), ("bar", 1)]
+    assert get_tab_widths("\tfoo\t\t") == [("", 4), ("foo", 1), ("", 4)]
+    assert get_tab_widths("\t木foo\t木\t\t") == [("", 4), ("木foo", 3), ("木", 2), ("", 4)]
