@@ -5,6 +5,8 @@ import asyncio
 import pytest
 
 from textual.app import App, ComposeResult
+from textual.message import Message
+from textual.message_pump import MessagePump
 from textual.reactive import Reactive, TooManyComputesError, reactive, var
 from textual.widget import Widget
 
@@ -705,3 +707,120 @@ async def test_external_watch_init_does_not_propagate_to_externals() -> None:
         assert logs == ["first", "second"]
         app.query_one(SomeWidget).test_var = 73
         assert logs == ["first", "second", "first", "second"]
+
+
+async def test_message_sender_from_reactive() -> None:
+    """Test that the sender of a message comes from the reacting widget."""
+
+    message_senders: list[MessagePump | None] = []
+
+    class TestWidget(Widget):
+        test_var: var[int] = var(0, init=False)
+
+        class TestMessage(Message):
+            pass
+
+        def watch_test_var(self) -> None:
+            self.post_message(self.TestMessage())
+
+        def make_reaction(self) -> None:
+            self.test_var += 1
+
+    class TestContainer(Widget):
+        def compose(self) -> ComposeResult:
+            yield TestWidget()
+
+        def on_test_widget_test_message(self, event: TestWidget.TestMessage) -> None:
+            nonlocal message_senders
+            message_senders.append(event._sender)
+
+    class TestApp(App[None]):
+        def compose(self) -> ComposeResult:
+            yield TestContainer()
+
+    async with TestApp().run_test() as pilot:
+        assert message_senders == []
+        pilot.app.query_one(TestWidget).make_reaction()
+        await pilot.pause()
+        assert message_senders == [pilot.app.query_one(TestWidget)]
+
+
+async def test_mutate_reactive() -> None:
+    """Test explicitly mutating reactives"""
+
+    watched_names: list[list[str]] = []
+
+    class TestWidget(Widget):
+        names: reactive[list[str]] = reactive(list)
+
+        def watch_names(self, names: list[str]) -> None:
+            watched_names.append(names.copy())
+
+    class TestApp(App):
+        def compose(self) -> ComposeResult:
+            yield TestWidget()
+
+    app = TestApp()
+    async with app.run_test():
+        widget = app.query_one(TestWidget)
+        # watch method called on startup
+        assert watched_names == [[]]
+
+        # Mutate the list
+        widget.names.append("Paul")
+        # No changes expected
+        assert watched_names == [[]]
+        # Explicitly mutate the reactive
+        widget.mutate_reactive(TestWidget.names)
+        # Watcher will be invoked
+        assert watched_names == [[], ["Paul"]]
+        # Make further modifications
+        widget.names.append("Jessica")
+        widget.names.remove("Paul")
+        # No change expected
+        assert watched_names == [[], ["Paul"]]
+        # Explicit mutation
+        widget.mutate_reactive(TestWidget.names)
+        # Watcher should be invoked
+        assert watched_names == [[], ["Paul"], ["Jessica"]]
+
+
+async def test_mutate_reactive_data_bind() -> None:
+    """https://github.com/Textualize/textual/issues/4825"""
+
+    # Record mutations to TestWidget.messages
+    widget_messages: list[list[str]] = []
+
+    class TestWidget(Widget):
+        messages: reactive[list[str]] = reactive(list, init=False)
+
+        def watch_messages(self, names: list[str]) -> None:
+            widget_messages.append(names.copy())
+
+    class TestApp(App):
+        messages: reactive[list[str]] = reactive(list, init=False)
+
+        def compose(self) -> ComposeResult:
+            yield TestWidget().data_bind(TestApp.messages)
+
+    app = TestApp()
+    async with app.run_test():
+        test_widget = app.query_one(TestWidget)
+        assert widget_messages == [[]]
+        assert test_widget.messages == []
+
+        # Should be the same instance
+        assert app.messages is test_widget.messages
+
+        # Mutate app
+        app.messages.append("foo")
+        # Mutations aren't detected
+        assert widget_messages == [[]]
+        assert app.messages == ["foo"]
+        assert test_widget.messages == ["foo"]
+        # Explicitly mutate app reactive
+        app.mutate_reactive(TestApp.messages)
+        # Mutating app, will also invoke watchers on any data binds
+        assert widget_messages == [[], ["foo"]]
+        assert app.messages == ["foo"]
+        assert test_widget.messages == ["foo"]
